;;; blkshape.s - Data tables relating to pieces in
;   LJ65 game for NES

;;; Copyright (C) 2003-2009 Damian Yerrick
;
;   This program is free software; you can redistribute it and/or
;   modify it under the terms of the GNU General Public License
;   as published by the Free Software Foundation; either version 3
;   of the License, or (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to 
;     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;     Boston, MA  02111-1307, USA.
;
;   Visit http://www.pineight.com/ for more information.

.segment "RODATA"
;when space conservation becomes important, 
;consider redoing the piece shapes in a "packed" fashion
;it'd save 192 bytes

.global xshapes, yshapes, nextshapes
.global fallencolors, darkcolors, segaColorScramble

.align 256

xshapes:
  .byt 0,1,2,3, 2,2,2,2, 3,2,1,0, 1,1,1,1 ; I
  .byt 0,1,2,0, 1,1,1,2, 2,1,0,2, 1,1,1,0 ; J
  .byt 0,1,2,2, 1,1,1,2, 2,1,0,0, 1,1,1,0 ; L
  .byt 1,1,2,2, 1,2,2,1, 2,2,1,1, 2,1,1,2 ; O
  .byt 0,1,1,2, 1,1,2,2, 2,1,1,0, 1,1,0,0 ; S
  .byt 0,1,2,1, 1,1,1,2, 2,1,0,1, 1,1,1,0 ; T
  .byt 0,1,1,2, 2,2,1,1, 2,1,1,0, 0,0,1,1 ; Z

yshapes:
  .byt 2,2,2,2, 3,2,1,0, 1,1,1,1, 0,1,2,3 ;    I
  .byt 2,2,2,3, 3,2,1,3, 2,2,2,1, 1,2,3,1 ;     J
  .byt 2,2,2,3, 3,2,1,1, 2,2,2,1, 1,2,3,3 ;     L
  .byt 2,3,3,2, 3,3,2,2, 3,2,2,3, 3,3,2,2 ;      O
  .byt 2,2,3,3, 3,2,2,1, 2,2,1,1, 1,2,2,3 ;     S
  .byt 2,2,2,3, 3,2,1,2, 2,2,2,1, 1,2,3,2 ;     T
  .byt 3,3,2,2, 3,2,2,1, 1,1,2,2, 1,2,2,3 ;     Z

nextshapes:
;                   0145
;       76543210    2367
  .byt %00110011 ; I
  .byt %01010011 ; J
  .byt %00010111 ; L
  .byt %01011010 ; O
  .byt %00011110 ; S
  .byt %00011011 ; T
  .byt %01001011 ; Z

fallencolors:
  .byt $81,$82,$83,$84,$85,$86,$87
  ;      I,  J,  L,  O,  S,  T,  Z

darkcolors:
  .byt $1C  ; I - cyan
  .byt $02  ; J - blue
  .byt $17  ; L - orange 
  .byt $18  ; O - yellow
  .byt $1A  ; S - green
  .byt $04  ; T - purple
  .byt $06  ; Z - red

segaColorScramble:
  .byt 6, 1, 2, 3, 5, 0, 4

; title screen sprite data
title_sprites_x:
  .byt 112,128,112,128  ;O
  .byt  96,112,128,144  ;I
  .byt  16, 16, 16, 32  ;L
  .byt 104,120,136,120  ;T
title_sprites_y:
  .byt  15, 15, 31, 31  ;O
  .byt  63, 63, 63, 63  ;I
  .byt  95,111,127,127  ;L
  .byt 143,143,143,159  ;T
title_pal:
  .byt $0F,$00,$10,$30
  .byt $0F,$06,$16,$26  ;Z
  .byt $0F,$02,$12,$22  ;J
  .byt $0F,$1a,$2a,$3a  ;S
  .byt $0F,$18,$28,$38  ;O
  .byt $0F,$1C,$2C,$3C  ;I
  .byt $0F,$17,$27,$37  ;L
  .byt $0F,$04,$14,$24  ;T

.segment "CODE"
.import OAM
.export build_title_oam

PPUADDR = $2006
PPUDATA = $2007

.proc build_title_oam
  ldx #$3F
  stx PPUADDR
  ldx #0
  stx PPUADDR
palloop:
  lda title_pal,x
  sta PPUDATA
  inx
  cpx #32
  bcc palloop

  ldx #0
  ldy #15
loop:
  ; Y position
  lda title_sprites_y,y
  sta OAM,x
  sta OAM+64,x
  inx
  ; Tile number
  lda #$AC
  sta OAM,x
  ora #2
  sta OAM+64,x
  inx
  ; Palette number
  tya
  lsr a
  lsr a
  sta OAM,x
  sta OAM+64,x
  inx
  ; X position
  lda title_sprites_x,y
  sta OAM,x
  clc
  adc #$08
  sta OAM+64,x
  inx
  dey
  bpl loop
  lda #$F0
  ldx #128
clear_rest:
  sta OAM,x
  inx
  bne clear_rest

  rts
.endproc
