; rotation.s
; how pieces turn and kick off walls in LJ65

;;; Copyright (C) 2009 Damian Yerrick
;
;   This program is free software; you can redistribute it and/or
;   modify it under the terms of the GNU General Public License
;   as published by the Free Software Foundation; either version 3
;   of the License, or (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to 
;     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;     Boston, MA  02111-1307, USA.
;
;   Visit http://www.pineight.com/ for more information.


.export try_rotating
.import collision_check, start_sound
.importzp piece_x, piece_y, cur_flip, rot_dir, has_floor_kicked
.importzp cur_piece, rotation_system, cur_state
.importzp STATE_LANDED_PIECE

DEBUG_CENTRAL_COLUMN = 0
centralColumnOnly = $0006

; try_rotating
; input:
; x = player number
; piece_x,x, piece_y,x = current position of piece
; cur_flip,x = current rotation
; a = rotation direction (1: clockwise; 3: anti)
.proc try_rotating
  sta rot_dir
  lda cur_flip,x
  pha
  clc
  adc rot_dir
  and #$03
  sta cur_flip,x
  lda rotation_system,x
  beq not_ars
  jmp try_rotating_ars
not_ars:
; first try in place
  jsr collision_check
  beq @got_rotate

.if ::DEBUG_CENTRAL_COLUMN
  lda centralColumnOnly
  beq @not_centralColumnOnly
  ; if 6 is set, all collisions were in the central column
  lda #6
  jsr start_sound
@not_centralColumnOnly:
.endif

; try one space to the right
  inc piece_x,x
  jsr collision_check
  beq @got_rotate
  dec piece_x,x

; try one space to the left
  dec piece_x,x
  jsr collision_check
  beq @got_rotate
  inc piece_x,x

  lda has_floor_kicked,x
  bne @already_used_floor_kick
; try one space up
  inc piece_y,x
  jsr collision_check
  beq @got_floor_kick
  dec piece_y,x
@already_used_floor_kick:

; failed. un-rotate the piece.

  pla
  sta cur_flip,x
  jmp @no_rotate

@got_floor_kick:
  inc has_floor_kicked,x
@got_rotate:
  pla
  lda #0
  jsr start_sound
  ;jmp check_still_landed
@no_rotate:
  rts
.endproc


; try rotating the piece under ARS
;
; in:
; x: player number
; stack: [top] last_flip rts ...
; cur_flip: destination rotation state
.proc try_rotating_ars
  lda cur_piece,x
  bne sega_not_i

  ; handle I piece
  ; TAP rules: states 0 (up) and 1 (right) only, no kicks
  lda cur_flip,x
  and #$01
  sta cur_flip,x
  jsr collision_check
  beq pass
  jmp try_kicking_ars_i
pass:
  pla
  lda #0
  jsr start_sound
  ;jmp check_still_landed
  rts
sega_not_i:

  cmp #3
  bne sega_not_o
  ; Like in Lumines, O rotation always succeeds
  ; and never changes the falling/landed state.
  pla
  lda #0
  jmp start_sound
sega_not_o:

  ; current stack: ... rts last_flip
  pla
  tay
  lda piece_y,x
  pha
  tya
  pha

  ; current stack: ... rts last_y last_flip
  ; y = last_flip
  cpy #0
  beq not_currently_up
  dec piece_y,x
not_currently_up:  


  ; S and Z cannot rotate to a state that leaves the
  ; bottom center block empty.
  ; For S, this is state 1; for Z, this is state 3.
  ; Instead, they rotate to the opposite state.
  lda cur_piece,x
  cmp #4
  bne sega_not_s
  lda cur_flip,x
  cmp #1
  bne special_cases_done
  lda #3
  sta cur_flip,x
  bne special_cases_done
sega_not_s:

  cmp #6
  bne sega_not_z
  lda cur_flip,x
  cmp #3
  bne special_cases_done
  lda #1
  sta cur_flip,x
  bne special_cases_done
sega_not_z:
special_cases_done:

  lda cur_flip,x
  beq not_to_up
  inc piece_y,x
not_to_up:

  ; first try rotating in place
  jsr collision_check
  beq @got_rotate

  ; for JLSTZ to be able to kick, one of the colliding
  ; blocks has to be outside the central column.
  ; collision_check sets this to nonzero if this isn't true.
  lda centralColumnOnly
  bne @already_used_floor_kick

; try one space to the right
  inc piece_x,x
  jsr collision_check
  beq @got_rotate
  dec piece_x,x

; try one space to the left
  dec piece_x,x
  jsr collision_check
  beq @got_rotate
  inc piece_x,x

  ; in Ti, among JLSTZ, only the T piece
  ; can floor kick, and then only from up
  lda cur_piece,x
  cmp #5
  bne @already_used_floor_kick
  lda has_floor_kicked,x
  bne @already_used_floor_kick
; try one space up
  inc piece_y,x
  jsr collision_check
  beq @got_floor_kick
  dec piece_y,x
@already_used_floor_kick:

; failed. un-rotate the piece.

  pla
  sta cur_flip,x
  pla
  sta piece_y,x
  rts

@got_floor_kick:
  inc has_floor_kicked,x
@got_rotate:
  pla
  jmp pass

.endproc


;
; Tries wall- or floor-kicking an I piece using Ti-ARS rules.
; in:
; x = player number
; stack: [top] last_flip rts
;
.proc try_kicking_ars_i
need_i_kick:
  lda cur_flip,x
  and #1
  beq need_i_wall_kick
  
  ; Vertical I:
  ; If the piece has landed but not already floor kicked,
  ; then try up 1 cell, then try up 2 cells, then fail.
  lda has_floor_kicked,x
  bne i_flip_back
  lda cur_state,x
  cmp #STATE_LANDED_PIECE
  bne i_flip_back
  ; Up 1
  inc piece_y,x
  jsr collision_check
  beq got_i_floor_kick
  ; Up 2
  inc piece_y,x
  jsr collision_check
  beq got_i_floor_kick
  ; Move it back and fail
  dec piece_y,x
  dec piece_y,x
  jmp i_flip_back
  
got_i_floor_kick:
  inc has_floor_kicked,x
got_i_wall_kick:
  pla
  lda #0
  jsr start_sound
  ;jmp check_still_landed
  rts

need_i_wall_kick:

  ; Try one to the left, one to the right, or two to the right.
  ; It turns out that whether we try one to the left or one to the
  ; right first doesn't matter as long as two to the right is after
  ; one to the right.
  
  ; Left 1
  dec piece_x,x
  jsr collision_check
  beq got_i_wall_kick
  inc piece_x,x
  ; Right 1
  inc piece_x,x
  jsr collision_check
  beq got_i_wall_kick
  ; Right 2
  inc piece_x,x
  jsr collision_check
  beq got_i_wall_kick
  ; Move it back and fail
  dec piece_x,x
  dec piece_x,x

i_flip_back:
  pla
  sta cur_flip,x
  rts
.endproc
